@@ -11,6 +11,7 @@
  */
 
 #include <config.h>
+#include <unistd.h>
 
 #include "foobar-widget.h"
 
@@ -27,7 +28,7 @@
 #include "scroll-menu.h"
 #include "gwmh.h"
 #include "tasklist_icon.h"
-#include "scroll-menu.h"
+#include "multiscreen-stuff.h"
 
 #define SMALL_ICON_SIZE 20
 
@@ -39,8 +40,6 @@ extern GtkTooltips *panel_tooltips;
 static void foobar_widget_class_init	(FoobarWidgetClass	*klass);
 static void foobar_widget_init		(FoobarWidget		*foo);
 static void foobar_widget_realize	(GtkWidget		*w);
-static gboolean foobar_widget_configure_event (GtkWidget        *w,
-					       GdkEventConfigure *event);
 static void foobar_widget_destroy	(GtkObject		*o);
 static void foobar_widget_size_allocate	(GtkWidget		*w,
 					 GtkAllocation		*alloc);
@@ -51,8 +50,7 @@ static gboolean foobar_enter_notify	(GtkWidget *widget,
 static void append_task_menu (FoobarWidget *foo, GtkMenuBar *menu_bar);
 static void setup_task_menu (FoobarWidget *foo);
 
-static GtkWidget *das_global_foobar = NULL;
-static GtkWidget *clock_ebox = NULL;
+static GList *foobars = NULL;
 
 static GtkWindowClass *parent_class = NULL;
 
@@ -93,7 +91,6 @@ foobar_widget_class_init (FoobarWidgetClass *klass)
 	object_class->destroy = foobar_widget_destroy;
 
 	widget_class->realize = foobar_widget_realize;
-	widget_class->configure_event = foobar_widget_configure_event;
 	widget_class->size_allocate = foobar_widget_size_allocate;
 	widget_class->enter_notify_event = foobar_enter_notify;
 	widget_class->leave_notify_event = foobar_leave_notify;
@@ -228,12 +225,12 @@ append_gnome_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 		{ NULL, "" },
 		{ N_("Software (www)"),            N_("http://www.gnome.org/applist/list-martin.phtml"), GNOME_STOCK_PIXMAP_SAVE },
 		{ N_("Development (www)"),         N_("http://developer.gnome.org/"),                    "gnome-devel.png" },
-		{ N_("Bug Tracking System (www)"), N_("http://bugs.gnome.org/"),                         "bug-buddy.png" },
+		{ N_("Bug Tracking System (www)"), N_("http://bugzilla.gnome.org/"),                         "bug-buddy.png" },
 		{ NULL }
 	};
 	
 	
-	menu = scroll_menu_new ();
+	menu = hack_scroll_menu_new ();
 	
 	for (i=0; url[i][1]; i++)
 		gtk_menu_append (GTK_MENU (menu),
@@ -332,7 +329,7 @@ append_desktop_menu (GtkWidget *menu_bar)
 	};
 	GList *gmc_menu_items = NULL;
 
-	menu = scroll_menu_new ();
+	menu = hack_scroll_menu_new ();
 
 	for (i=0; arrange[i]; i+=2)
 		append_gmc_item (menu, _(arrange[i]), arrange[i+1]);
@@ -343,7 +340,7 @@ append_desktop_menu (GtkWidget *menu_bar)
 
 	add_tearoff (GTK_MENU (menu));
 
-	menu = scroll_menu_new ();
+	menu = hack_scroll_menu_new ();
 
 	gtk_menu_append (GTK_MENU (menu), item);
 
@@ -405,13 +402,21 @@ append_folder_menu (GtkWidget *menu_bar, const char *label,
 		: gnome_util_home_file (path);
 
 	if (real_path == NULL) {
-		g_warning (_("can't fine real path"));
+		g_warning (_("can't find real path"));
 		return NULL;
 	}
 
-	menu = create_fake_menu_at (real_path, FALSE, FALSE,
-				    label, NULL, FALSE);
+	menu = create_fake_menu_at (real_path,
+				    FALSE /* applets */,
+				    FALSE /* launcher_add */,
+				    FALSE /* favourites_add */,
+				    label /* dir_name */,
+				    NULL /* pixmap_name */,
+				    FALSE /* title */);
 	g_free (real_path);
+	if (path != NULL && strcmp (path, "apps") == 0)
+		/* This will add the add submenu thingie */
+		start_favourites_menu (menu, TRUE /* fake_submenus */);
 
 	if (menu == NULL) {
 		g_warning (_("menu wasn't created"));
@@ -464,7 +469,8 @@ update_clock (FoobarWidget *foo)
 		}
 		hour[sizeof(hour)-1] = '\0'; /* just for sanity */
 
-		gtk_tooltips_set_tip (panel_tooltips, clock_ebox, hour, NULL);
+		gtk_tooltips_set_tip (panel_tooltips, foo->clock_ebox,
+				      hour, NULL);
 
 		day = das_tm->tm_mday;
 	}
@@ -498,11 +504,12 @@ timeout_cb (gpointer data)
 static void
 set_fooclock_format (GtkWidget *w, char *format)
 {
-	if ( ! IS_FOOBAR_WIDGET (das_global_foobar))
-		return;
+	GList *li;
 
-	foobar_widget_set_clock_format (FOOBAR_WIDGET (das_global_foobar),
-					_(format));
+	for (li = foobars; li != NULL; li = li->next) {
+		foobar_widget_set_clock_format (FOOBAR_WIDGET (li->data),
+						_(format));
+	}
 }
 
 static void
@@ -529,10 +536,21 @@ append_format_item (GtkWidget *menu, const char *format)
 			    (gpointer)format);
 }
 
+static void
+set_time_cb (GtkWidget *menu_item, char *path)
+{
+	char *v[2] = { path };
+	
+	if (gnome_execute_async (g_get_home_dir (), 1, v) < 0)
+		panel_error_dialog (_("Could not call time-admin\n"
+						  "Perhaps time-admin is not installed"));
+}
+
 static GtkWidget *
 append_clock_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 {
 	GtkWidget *item, *menu, *menu2;
+	gchar *time_admin_path;
 	int i;
 	const char *cals[] = { 
 		N_("Today"),      "dayview",
@@ -549,7 +567,7 @@ append_clock_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 		NULL
 	};
 
-	menu = scroll_menu_new ();
+	menu = hack_scroll_menu_new ();
 	
 #if 0 /* put back when evolution can do this */
 	item = gtk_menu_item_new_with_label (_("Add appointement..."));
@@ -558,12 +576,21 @@ append_clock_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 	add_menu_separator (menu);
 #endif
 
+	time_admin_path = gnome_is_program_in_path ("time-admin");
+	if (time_admin_path) {
+		item = gtk_menu_item_new_with_label (_("Set Time"));
+		gtk_signal_connect (GTK_OBJECT (item), "activate",
+						GTK_SIGNAL_FUNC (set_time_cb), time_admin_path);
+		gtk_menu_append (GTK_MENU (menu), item);
+		add_menu_separator (menu);
+	}
+
 	for (i=0; cals[i]; i+=2)
 		append_gnomecal_item (menu, _(cals[i]), cals[i+1]);
 
 	add_menu_separator (menu);
 
-	menu2 = scroll_menu_new ();
+	menu2 = hack_scroll_menu_new ();
 	for (i=0; formats[i]; i++)
 		append_format_item (menu2, formats[i]);
 
@@ -576,10 +603,11 @@ append_clock_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 	add_tearoff (GTK_MENU (menu));
 
 	item = gtk_menu_item_new ();
+
 	foo->clock_label = gtk_label_new ("");
 	foo->clock_timeout = gtk_timeout_add (1000, timeout_cb, foo);
 
-	clock_ebox = item;
+	foo->clock_ebox = item;
 	gtk_container_add (GTK_CONTAINER (item), foo->clock_label);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
 
@@ -591,12 +619,12 @@ append_clock_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 void
 foobar_widget_global_set_clock_format (const char *format)
 {
-	if (das_global_foobar == NULL ||
-	    ! IS_FOOBAR_WIDGET (das_global_foobar))
-		return;
+	GList *li;
 
-	foobar_widget_set_clock_format (FOOBAR_WIDGET (das_global_foobar),
-					format);
+	for (li = foobars; li != NULL; li = li->next) {
+		foobar_widget_set_clock_format (FOOBAR_WIDGET (li->data),
+						format);
+	}
 }
 
 void
@@ -617,8 +645,11 @@ foobar_widget_update_winhints (FoobarWidget *foo)
 	if ( ! foo->compliant_wm)
 		return;
 
-	gdk_window_set_hints (w->window, 0, 0, 
-			      0, 0, 0, 0, GDK_HINT_POS);
+	xstuff_set_pos_size (w->window,
+			     multiscreen_x (foo->screen),
+			     multiscreen_y (foo->screen),
+			     w->allocation.width,
+			     w->allocation.height);
 
 	gnome_win_hints_set_expanded_size (w, 0, 0, 0, 0);
 	gdk_window_set_decorations (w->window, 0);
@@ -652,21 +683,6 @@ foobar_widget_realize (GtkWidget *w)
 	setup_task_menu (FOOBAR_WIDGET (w));
 }
 
-static gboolean
-foobar_widget_configure_event (GtkWidget *w, GdkEventConfigure *event)
-{
-	g_return_val_if_fail (IS_FOOBAR_WIDGET (w), FALSE);
-
-	if (event->x != 0 ||
-	    event->y != 0)
-		gdk_window_move (w->window, 0, 0);
-
-	if (GTK_WIDGET_CLASS (parent_class)->configure_event != NULL)
-		return GTK_WIDGET_CLASS (parent_class)->configure_event (w, event);
-	else
-		return FALSE;
-}
-
 static void
 programs_menu_to_display(GtkWidget *menu)
 {
@@ -688,7 +704,7 @@ programs_menu_to_display(GtkWidget *menu)
 static void
 set_the_task_submenu (FoobarWidget *foo, GtkWidget *item)
 {
-	foo->task_menu = scroll_menu_new ();
+	foo->task_menu = hack_scroll_menu_new ();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), foo->task_menu);
 	/*g_message ("setting...");*/
 }
@@ -697,6 +713,8 @@ static void
 focus_task (GtkWidget *w, GwmhTask *task)
 {
 	gwmh_desk_set_current_area (task->desktop, task->harea, task->varea);
+	if (GWMH_TASK_ICONIFIED (task)) 
+		gwmh_task_deiconify (task);
 	gwmh_task_show  (task);
 	gwmh_task_raise (task);
 	gwmh_task_focus (task);
@@ -716,10 +734,25 @@ add_task (GwmhTask *task, FoobarWidget *foo)
 
 	if (GWMH_TASK_SKIP_WINLIST (task))
 		return;
-	/*g_message ("task: %s", task->name);*/
-	slen = strlen (task->name);
-	if (slen > 43)
-		title = g_strdup_printf ("%.20s...%s", task->name, task->name+slen-20);
+	if (task->name != NULL) {
+		slen = strlen (task->name);
+		if (slen > 443)
+			title = g_strdup_printf ("%.420s...%s", task->name, task->name+slen-20);
+		else
+			title = g_strdup (task->name);
+	} else {
+		/* Translators: Task with no name, should not really happen, so
+		 * this should signal that the panel is confused by this task
+		 * (thus question marks) */
+		title = g_strdup (_("???"));
+	}
+
+	if (GWMH_TASK_ICONIFIED (task)) {
+		char *tmp = title;
+		title = g_strdup_printf ("[%s]", title);
+		g_free (tmp);
+	}
+	
 	item = gtk_pixmap_menu_item_new ();
 	pixmap = get_task_icon (task, GTK_WIDGET (foo));
 	if (pixmap != NULL) {
@@ -728,7 +761,7 @@ add_task (GwmhTask *task, FoobarWidget *foo)
 						 pixmap);
 	}
 
-	label = gtk_label_new (title ? title : task->name);
+	label = gtk_label_new (title);
 	g_free (title);
 
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
@@ -755,14 +788,25 @@ create_task_menu (GtkWidget *w, gpointer data)
 {
 	FoobarWidget *foo = FOOBAR_WIDGET (data);
 	GList *tasks = gwmh_task_list_get ();
+	GList *list;
+	GtkWidget *separator;
 
 	/*g_message ("creating...");*/
 	foo->tasks = g_hash_table_new (g_direct_hash, g_direct_equal);
 
-	add_menu_separator (foo->task_menu);
+	separator = add_menu_separator (foo->task_menu);
 
 	g_list_foreach (tasks, (GFunc)add_task, foo);
 
+	list = g_list_last (GTK_MENU_SHELL (foo->task_menu)->children);
+
+	if (list != NULL &&
+	    separator == list->data) {
+		/* if the separator is the last item wipe it.
+		 * We leave it as the first though */
+		gtk_widget_destroy (separator);
+	}
+
 	/* Owen: don't read the next line */
 	GTK_MENU_SHELL (GTK_MENU_ITEM (w)->submenu)->active = 1;
 	our_gtk_menu_position (GTK_MENU (GTK_MENU_ITEM (w)->submenu));
@@ -826,12 +870,16 @@ set_das_pixmap (FoobarWidget *foo, GwmhTask *task)
 	if (!GTK_WIDGET_REALIZED (foo))
 		return;
 
+	foo->icon_task = NULL;
+
 	if (foo->task_pixmap != NULL)
 		gtk_widget_destroy (foo->task_pixmap);
 	foo->task_pixmap = NULL;
 
-	if (task != NULL)
+	if (task != NULL) {
 		foo->task_pixmap = get_task_icon (task, GTK_WIDGET (foo));
+		foo->icon_task = task;
+	}
 
 	if (foo->task_pixmap == NULL) {
 		foo->task_pixmap = get_default_pixmap ();
@@ -844,7 +892,8 @@ set_das_pixmap (FoobarWidget *foo, GwmhTask *task)
 }
 
 static gboolean
-task_notify (gpointer data, GwmhTask *task,
+task_notify (gpointer data,
+	     GwmhTask *task,
 	     GwmhTaskNotifyType ntype,
 	     GwmhTaskInfoMask imask)
 {
@@ -853,19 +902,32 @@ task_notify (gpointer data, GwmhTask *task,
 
 	switch (ntype) {
 	case GWMH_NOTIFY_INFO_CHANGED:
-		if (imask & GWMH_TASK_INFO_FOCUSED &&
-		    GWMH_TASK_FOCUSED (task)) {
-			set_das_pixmap (foo, task);
-		}
 		if (imask & GWMH_TASK_INFO_WM_HINTS &&
-		    GWMH_TASK_FOCUSED (task))
+		    GWMH_TASK_FOCUSED (task)) {
+			/* icon might have changed */
 			set_das_pixmap (foo, task);
+		} else if (imask & GWMH_TASK_INFO_FOCUSED) {
+			if (GWMH_TASK_FOCUSED (task) &&
+			    foo->icon_task != task) {
+				/* Focused and not set in the top thingie,
+				 * so setup */
+				set_das_pixmap (foo, task);
+			} else if ( ! GWMH_TASK_FOCUSED (task) &&
+				   task == foo->icon_task) {
+				/* Just un-focused and currently the
+				 * icon_task, so set the pixmap to
+				 * the default (nothing) */
+				set_das_pixmap (foo, NULL);
+			}
+		}
 		break;
 	case GWMH_NOTIFY_NEW:
 		if (foo->tasks != NULL)
 			add_task (task, foo);
 		break;
 	case GWMH_NOTIFY_DESTROY:
+		if (task == foo->icon_task)
+			set_das_pixmap (foo, NULL);
 		/* FIXME: Whoa; leak? */
 		if (foo->tasks != NULL) {
 			item = g_hash_table_lookup (foo->tasks, task);
@@ -874,7 +936,7 @@ task_notify (gpointer data, GwmhTask *task,
 				gtk_widget_hide (item);
 			} else {
 				g_warning ("Could not find item for task '%s'",
-					   task->name);
+					   sure_string (task->name));
 			}
 		}
 		break;
@@ -938,6 +1000,14 @@ foobar_widget_init (FoobarWidget *foo)
 	/*GtkWidget *align;*/
 	gint flags;
 
+	foo->screen = 0;
+
+	foo->task_item = NULL;
+	foo->task_menu = NULL;
+	foo->task_pixmap = NULL;
+	foo->task_bin = NULL;
+	foo->icon_task = NULL;
+
 	foo->clock_format = g_strdup (_("%H:%M"));
 	foo->clock_timeout = 0;
 	foo->clock_label = NULL;
@@ -955,9 +1025,11 @@ foobar_widget_init (FoobarWidget *foo)
 	gtk_signal_connect (GTK_OBJECT (foo), "delete_event",
 			    GTK_SIGNAL_FUNC (gtk_true), NULL);
 
-	gtk_widget_set_uposition (GTK_WIDGET (foo), 0, 0);
+	gtk_widget_set_uposition (GTK_WIDGET (foo),
+				  multiscreen_x (foo->screen),
+				  multiscreen_y (foo->screen));
 	gtk_widget_set_usize (GTK_WIDGET (foo),
-			      gdk_screen_width (), -2);
+			      multiscreen_width (foo->screen), -2);
 
 	foo->ebox = gtk_event_box_new ();
 	foo->hbox = gtk_hbox_new (FALSE, 0);
@@ -977,7 +1049,8 @@ foobar_widget_init (FoobarWidget *foo)
 				      GTK_SHADOW_NONE);
 	
 	
-	menuitem = pixmap_menu_item_new (_("Programs"), GNOME_STOCK_MENU_ABOUT);
+	menuitem = pixmap_menu_item_new (_("Programs"),
+					 "gnome-logo-icon-transparent.png");
 	flags = (get_default_menu_flags() & 
 		 ~(MAIN_MENU_SYSTEM_SUB | MAIN_MENU_USER | MAIN_MENU_USER_SUB |
 		   MAIN_MENU_PANEL | MAIN_MENU_PANEL_SUB | MAIN_MENU_DESKTOP |
@@ -1058,9 +1131,7 @@ foobar_widget_destroy (GtkObject *o)
 {
 	FoobarWidget *foo = FOOBAR_WIDGET (o);
 
-	/* Just sanity */
-	if ((gpointer)das_global_foobar == (gpointer)foo)
-		das_global_foobar = NULL;
+	foobars = g_list_remove (foobars, foo);
 
 	if (foo->clock_timeout != 0)
 		gtk_timeout_remove (foo->clock_timeout);
@@ -1091,52 +1162,91 @@ foobar_widget_size_allocate (GtkWidget *w, GtkAllocation *alloc)
 		GTK_WIDGET_CLASS (parent_class)->size_allocate (w, alloc);
 
 	if (GTK_WIDGET_REALIZED (w)) {
+		FoobarWidget *foo = FOOBAR_WIDGET (w);
+		xstuff_set_pos_size (w->window,
+				     multiscreen_x (foo->screen),
+				     multiscreen_y (foo->screen),
+				     alloc->width,
+				     alloc->height);
+
 		g_slist_foreach (panel_list, queue_panel_resize, NULL);
-		basep_border_queue_recalc ();
+		basep_border_queue_recalc (foo->screen);
 	}
 }
 
 GtkWidget *
-foobar_widget_new (void)
+foobar_widget_new (int screen)
 {
-	g_return_val_if_fail (das_global_foobar == NULL, NULL);
+	FoobarWidget *foo;
 
-	das_global_foobar = gtk_type_new (TYPE_FOOBAR_WIDGET);
+	g_return_val_if_fail (screen >= 0, NULL);
 
-	return das_global_foobar;
+	if (foobar_widget_exists (screen))
+		return NULL;
+
+	foo = gtk_type_new (TYPE_FOOBAR_WIDGET);
+
+	foo->screen = screen;
+	gtk_widget_set_uposition (GTK_WIDGET (foo),
+				  multiscreen_x (foo->screen),
+				  multiscreen_y (foo->screen));
+	gtk_widget_set_usize (GTK_WIDGET (foo),
+			      multiscreen_width (foo->screen), -2);
+
+	foobars = g_list_prepend (foobars, foo);
+
+	return GTK_WIDGET (foo);
 }
 
 gboolean
-foobar_widget_exists (void)
+foobar_widget_exists (int screen)
 {
-	return (das_global_foobar != NULL);
+	GList *li;
+
+	for (li = foobars; li != NULL; li = li->next) {
+		FoobarWidget *foo = li->data;
+
+		if (foo->screen == screen)
+			return TRUE;
+	}
+	return FALSE;
 }
 
 void
 foobar_widget_force_menu_remake (void)
 {
 	FoobarWidget *foo;
-	if (das_global_foobar == NULL)
-		return;
+	GList *li;
 
-	foo = FOOBAR_WIDGET(das_global_foobar);
+	for (li = foobars; li != NULL; li = li->next) {
+		foo = FOOBAR_WIDGET(li->data);
 
-	if (foo->programs != NULL)
-		gtk_object_set_data (GTK_OBJECT(foo->programs),
-				     "need_reread", GINT_TO_POINTER(1));
-	if (foo->settings != NULL)
-		gtk_object_set_data (GTK_OBJECT(foo->settings),
-				     "need_reread", GINT_TO_POINTER(1));
-	if (foo->favorites != NULL)
-		gtk_object_set_data (GTK_OBJECT(foo->favorites),
-				     "need_reread", GINT_TO_POINTER(1));
+		if (foo->programs != NULL)
+			gtk_object_set_data (GTK_OBJECT(foo->programs),
+					     "need_reread", GINT_TO_POINTER(1));
+		if (foo->settings != NULL)
+			gtk_object_set_data (GTK_OBJECT(foo->settings),
+					     "need_reread", GINT_TO_POINTER(1));
+		if (foo->favorites != NULL)
+			gtk_object_set_data (GTK_OBJECT(foo->favorites),
+					     "need_reread", GINT_TO_POINTER(1));
+	}
 }
 
 gint
-foobar_widget_get_height (void)
+foobar_widget_get_height (int screen)
 {
-	return (das_global_foobar && GTK_WIDGET_REALIZED (das_global_foobar)) 
-		? das_global_foobar->allocation.height : 0; 
+	GList *li;
+
+	g_return_val_if_fail (screen >= 0, 0);
+
+	for (li = foobars; li != NULL; li = li->next) {
+		FoobarWidget *foo = FOOBAR_WIDGET(li->data);
+
+		if (foo->screen == screen)
+			return GTK_WIDGET (foo)->allocation.height;
+	}
+	return 0; 
 }
 
 static void
