@@ -15,6 +15,7 @@ struct _MenuFinfo {
 	char *menudir;
 	gboolean applets;
 	gboolean launcher_add;
+	gboolean favourites_add;
 	char *dir_name;
 	char *pixmap_name;	
 	gboolean fake_menu;
@@ -71,6 +72,7 @@ FileRec * fr_get_dir(const char *mdir);
 gboolean fr_is_subdir (const char *dir, const char *superdir, int superdir_len);
 char * fr_get_mergedir (const char *dir);
 void fr_force_reread(void);
+GSList* fr_get_all_dirs (void);
 
 void init_fr_chunks (void);
 
